
package com.example.android.cubesampleview;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.GLUtils;
import android.opengl.Matrix;
import android.os.SystemClock;
import android.util.Log;

class CubeSampleRenderer implements GLSurfaceView.Renderer {

	public static int s_touch_x = 0;
	public static int s_touch_y = 0;

    public CubeSampleRenderer(Context context) {
        mContext = context;
        mTriangleVertices = ByteBuffer.allocateDirect(mTriangleVerticesData.length * FLOAT_SIZE_BYTES).order(ByteOrder.nativeOrder()).asFloatBuffer();
        mTriangleVertices.put(mTriangleVerticesData).position(0);
    }

    private static int   S_SCREEN_BASE_RES_W = 480;
	private static int   S_SCREEN_BASE_RES_H = 800;
	private static float S_SCREEN_WIDTH  = S_SCREEN_BASE_RES_W * 1.0f;
	private static float S_SCREEN_HEIGHT = S_SCREEN_BASE_RES_H * 1.0f;
	private static float S_SCREEN_ASPECT_RATIO = S_SCREEN_HEIGHT / S_SCREEN_WIDTH;

	private static float S_FRUSTUM_W = S_SCREEN_WIDTH / 512.0f;
	private static float S_FRUSTUM_H = S_FRUSTUM_W * S_SCREEN_ASPECT_RATIO;
	private static float S_FRUSTUM_ZN = 5.0f;
	private static float S_FRUSTUM_ZF = 12.0f;

    public void onSurfaceChanged(GL10 glUnused, int width, int height)
    {
        Log.d("[SMGAL]", "screen size changed (" + width + "," + height + ")");

    	S_SCREEN_BASE_RES_W = width;
    	S_SCREEN_BASE_RES_H = height;
    	S_SCREEN_WIDTH  = S_SCREEN_BASE_RES_W * 1.0f;
    	S_SCREEN_HEIGHT = S_SCREEN_BASE_RES_H * 1.0f;
    	S_SCREEN_ASPECT_RATIO = S_SCREEN_HEIGHT / S_SCREEN_WIDTH;

    	S_FRUSTUM_W = S_SCREEN_WIDTH / 512.0f;
    	S_FRUSTUM_H = S_FRUSTUM_W * S_SCREEN_ASPECT_RATIO;
    	S_FRUSTUM_ZN = 5.0f;
    	S_FRUSTUM_ZF = 12.0f;

    	// Ignore the passed-in GL10 interface, and use the GLES20
        // class's static methods instead.
        GLES20.glViewport(0, 0, width, height);
        
        Matrix.frustumM(mPMatrix, 0, -S_FRUSTUM_W, S_FRUSTUM_W, -S_FRUSTUM_H, S_FRUSTUM_H, S_FRUSTUM_ZN, S_FRUSTUM_ZF);
    }

    public void onDrawFrame(GL10 glUnused) {
        // Ignore the passed-in GL10 interface, and use the GLES20
        // class's static methods instead.
        GLES20.glClearColor(0.3f, 0.2f, 0.6f, 1.0f);
        GLES20.glClear( GLES20.GL_DEPTH_BUFFER_BIT | GLES20.GL_COLOR_BUFFER_BIT);
        GLES20.glUseProgram(mProgram);
        checkGlError("glUseProgram");

        GLES20.glActiveTexture(GLES20.GL_TEXTURE0);

        //
        mTriangleVertices.position(TRIANGLE_VERTICES_DATA_POS_OFFSET);
        GLES20.glVertexAttribPointer(maPositionHandle, 3, GLES20.GL_FLOAT, false,
                TRIANGLE_VERTICES_DATA_STRIDE_BYTES, mTriangleVertices);
        checkGlError("glVertexAttribPointer maPosition");

        GLES20.glEnableVertexAttribArray(maPositionHandle);
        checkGlError("glEnableVertexAttribArray maPositionHandle");

        //
        mTriangleVertices.position(TRIANGLE_VERTICES_DATA_UV_OFFSET);
        GLES20.glVertexAttribPointer(maTextureHandle, 2, GLES20.GL_FLOAT, false,
                TRIANGLE_VERTICES_DATA_STRIDE_BYTES, mTriangleVertices);
        checkGlError("glVertexAttribPointer maTextureHandle");

        GLES20.glEnableVertexAttribArray(maTextureHandle);
        checkGlError("glEnableVertexAttribArray maTextureHandle");

        //
        mTriangleVertices.position(TRIANGLE_VERTICES_DATA_COLOR_OFFSET);
        GLES20.glVertexAttribPointer(maColorHandle, 4, GLES20.GL_FLOAT, false,
                TRIANGLE_VERTICES_DATA_STRIDE_BYTES, mTriangleVertices);
        checkGlError("glVertexAttribPointer maColorHandle");

        GLES20.glEnableVertexAttribArray(maColorHandle);
        checkGlError("glEnableVertexAttribArray maColorHandle");

        {
	        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mTextureID[0]);
	        
	        long  time      = SystemClock.uptimeMillis() % 4000L;
	        float angle     = 0.090f * ((int) time);
	        float revised_x = (s_touch_x - S_SCREEN_BASE_RES_W/2) * S_FRUSTUM_W / (S_SCREEN_BASE_RES_W/2);
	        float revised_y = (s_touch_y - S_SCREEN_BASE_RES_H/2) * S_FRUSTUM_H / (S_SCREEN_BASE_RES_H/2);

	        Matrix.setIdentityM(mMMatrix, 0);
	        Matrix.translateM(mMMatrix, 0, revised_x, revised_y, 0.0f);
	        Matrix.rotateM(mMMatrix, 0, angle, 0.7f, 0.3f, 0.5f);
	        Matrix.multiplyMM(mMVPMatrix, 0, mVMatrix, 0, mMMatrix, 0);
	        Matrix.multiplyMM(mMVPMatrix, 0, mPMatrix, 0, mMVPMatrix, 0);
	
	        GLES20.glUniformMatrix4fv(muMVPMatrixHandle, 1, false, mMVPMatrix, 0);
	        GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, 6*6);
	        checkGlError("glDrawArrays");
        }
    }

    public void onSurfaceCreated(GL10 glUnused, EGLConfig config) {
        // Ignore the passed-in GL10 interface, and use the GLES20
        // class's static methods instead.
        mProgram = createProgram(mVertexShader, mFragmentShader);
        if (mProgram == 0) {
            return;
        }
        maPositionHandle = GLES20.glGetAttribLocation(mProgram, "aPosition");
        checkGlError("glGetAttribLocation aPosition");
        if (maPositionHandle == -1) {
            throw new RuntimeException("Could not get attrib location for aPosition");
        }
        maTextureHandle = GLES20.glGetAttribLocation(mProgram, "aTextureCoord");
        checkGlError("glGetAttribLocation aTextureCoord");
        if (maTextureHandle == -1) {
            throw new RuntimeException("Could not get attrib location for aTextureCoord");
        }

        maColorHandle = GLES20.glGetAttribLocation(mProgram, "aColor");
        checkGlError("glGetAttribLocation aColor");
        if (maColorHandle == -1) {
            throw new RuntimeException("Could not get attrib location for aColor");
        }

        muMVPMatrixHandle = GLES20.glGetUniformLocation(mProgram, "uMVPMatrix");
        checkGlError("glGetUniformLocation uMVPMatrix");
        if (muMVPMatrixHandle == -1) {
            throw new RuntimeException("Could not get attrib location for uMVPMatrix");
        }

/*
        GLES20.glEnable(GLES20.GL_CULL_FACE);
        GLES20.glCullFace(GLES20.GL_FRONT);
*/
        GLES20.glEnable(GLES20.GL_DEPTH_TEST);

        GLES20.glEnable(GLES20.GL_BLEND);
        GLES20.glBlendFunc(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);

		mTextureID[0] = loadTexture(R.raw.aruku);
        Matrix.setLookAtM(mVMatrix, 0, 0, 0, -S_FRUSTUM_ZN*2.0f, 0f, 0f, 0f, 0f, -1.0f, 0.0f);
    }

    private int loadShader(int shaderType, String source) {
        int shader = GLES20.glCreateShader(shaderType);
        if (shader != 0) {
            GLES20.glShaderSource(shader, source);
            GLES20.glCompileShader(shader);
            int[] compiled = new int[1];
            GLES20.glGetShaderiv(shader, GLES20.GL_COMPILE_STATUS, compiled, 0);
            if (compiled[0] == 0) {
                Log.e(TAG, "Could not compile shader " + shaderType + ":");
                Log.e(TAG, GLES20.glGetShaderInfoLog(shader));
                GLES20.glDeleteShader(shader);
                shader = 0;
            }
        }
        return shader;
    }

    private int createProgram(String vertexSource, String fragmentSource) {
        int vertexShader = loadShader(GLES20.GL_VERTEX_SHADER, vertexSource);
        if (vertexShader == 0) {
            return 0;
        }

        int pixelShader = loadShader(GLES20.GL_FRAGMENT_SHADER, fragmentSource);
        if (pixelShader == 0) {
            return 0;
        }

        int program = GLES20.glCreateProgram();
        if (program != 0) {
            GLES20.glAttachShader(program, vertexShader);
            checkGlError("glAttachShader");
            GLES20.glAttachShader(program, pixelShader);
            checkGlError("glAttachShader");
            GLES20.glLinkProgram(program);
            int[] linkStatus = new int[1];
            GLES20.glGetProgramiv(program, GLES20.GL_LINK_STATUS, linkStatus, 0);
            if (linkStatus[0] != GLES20.GL_TRUE) {
                Log.e(TAG, "Could not link program: ");
                Log.e(TAG, GLES20.glGetProgramInfoLog(program));
                GLES20.glDeleteProgram(program);
                program = 0;
            }
        }
        return program;
    }

    private void checkGlError(String op) {
        int error;
        while ((error = GLES20.glGetError()) != GLES20.GL_NO_ERROR) {
            Log.e(TAG, op + ": glError " + error);
            throw new RuntimeException(op + ": glError " + error);
        }
    }

    private int loadTexture(int resourceId)
    {
    	return loadTexture(resourceId, false, 0);
    }
    
    private int loadTexture(int resourceId, boolean hasMaskingColor, int maskingColor)
	{
        int[] textures = new int[1];
        GLES20.glGenTextures(1, textures, 0);

		GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textures[0]);

		GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR);
		GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_LINEAR);

		GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, GLES20.GL_CLAMP_TO_EDGE);
		GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, GLES20.GL_CLAMP_TO_EDGE);

		Bitmap bitmap;
		
		{
			InputStream is = mContext.getResources().openRawResource(resourceId);

			Bitmap decoding_bitmap;

			try
			{
				decoding_bitmap = BitmapFactory.decodeStream(is);
				bitmap = decoding_bitmap.copy(Config.ARGB_8888, true);
				decoding_bitmap.recycle();
			}
			finally
			{
				try
				{
					is.close();
				}
				catch (IOException e)
				{
				}
			}
		}

		if (bitmap.getConfig() == Config.ARGB_8888)
		{
			if (hasMaskingColor)
			{
	        	int buffer_size = bitmap.getWidth() * bitmap.getHeight() * 4;
	        	int sBuffer[] = new int[buffer_size];
	    		IntBuffer sb = IntBuffer.wrap(sBuffer);
	    		bitmap.copyPixelsToBuffer(sb);
	    		
	    		for (int i = 0; i < buffer_size; i++)
	    		{
	    			if (sBuffer[i] == maskingColor)
	    			{
	    				sBuffer[i] = (int)(0x00000000);
	    			}
	    		}
	
	    		sb.rewind();
	    		bitmap.copyPixelsFromBuffer(sb);
			}
		}

		GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, bitmap, 0);
		bitmap.recycle();

		return textures[0];
	}
	
	private static final int FLOAT_SIZE_BYTES = 4;

	private static final int TRIANGLE_VERTICES_DATA_STRIDE_BYTES = 9 * FLOAT_SIZE_BYTES;
    private static final int TRIANGLE_VERTICES_DATA_POS_OFFSET   = 0;
    private static final int TRIANGLE_VERTICES_DATA_UV_OFFSET    = 3;
    private static final int TRIANGLE_VERTICES_DATA_COLOR_OFFSET = 5;

    private final float[] mTriangleVerticesData =
	{
		// X, Y, Z, U, V
		-1.0f, -1.0f, -1.0f, 0.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f, -1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		-1.0f,  1.0f, -1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		-1.0f,  1.0f, -1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f, -1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f,  1.0f, -1.0f, 1.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,

		-1.0f, -1.0f,  1.0f, 0.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		-1.0f,  1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f, -1.0f,  1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f, -1.0f,  1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		-1.0f,  1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f,  1.0f,  1.0f, 1.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,

		-1.0f, -1.0f, -1.0f, 0.0f, 0.0f,  1.0f,0.0f,0.0f,1.0f,
		-1.0f,  1.0f, -1.0f, 1.0f, 0.0f,  0.0f,1.0f,0.0f,1.0f,
		-1.0f, -1.0f,  1.0f, 0.0f, 1.0f,  0.0f,0.0f,1.0f,1.0f,
		-1.0f, -1.0f,  1.0f, 0.0f, 1.0f,  0.0f,0.0f,1.0f,1.0f,
		-1.0f,  1.0f, -1.0f, 1.0f, 0.0f,  0.0f,1.0f,0.0f,1.0f,
		-1.0f,  1.0f,  1.0f, 1.0f, 1.0f,  1.0f,0.0f,0.0f,1.0f,

		 1.0f, -1.0f, -1.0f, 0.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f, -1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f,  1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f,  1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f, -1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.0f,  1.0f,  1.0f, 1.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,

		 -1.0f, 1.0f, -1.0f, 0.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 -1.0f, 1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		  1.0f, 1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		  1.0f, 1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 -1.0f, 1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		  1.0f, 1.0f,  1.0f, 1.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,

		 -1.0f,-1.0f, -1.0f, 0.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		  1.0f,-1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 -1.0f,-1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 -1.0f,-1.0f,  1.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		  1.0f,-1.0f, -1.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		  1.0f,-1.0f,  1.0f, 1.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,

		// X, Y, Z, U, V
		-1.00f, -1.00f, -0.0f, 0.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.00f, -1.00f, -0.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		-1.00f,  1.00f, -0.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		-1.00f,  1.00f, -0.0f, 0.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.00f, -1.00f, -0.0f, 1.0f, 0.0f,  1.0f,1.0f,1.0f,1.0f,
		 1.00f,  1.00f, -0.0f, 1.0f, 1.0f,  1.0f,1.0f,1.0f,1.0f,
	};
    
    private FloatBuffer mTriangleVertices;

    private final String mVertexShader =
        "uniform   mat4  uMVPMatrix;\n"                +
        "attribute vec4  aPosition;\n"                 +
        "attribute vec2  aTextureCoord;\n"             +
        "attribute vec4  aColor;\n"                    +
        "varying   vec2  vTextureCoord;\n"             +
        "varying   vec4  vDiffuseColor;\n"             +
        "void main() {\n"                              +
        "   gl_Position   = uMVPMatrix * aPosition;\n" +
        "   vDiffuseColor = aColor;\n"                 +
        "   vTextureCoord = aTextureCoord;\n"          +
        "}\n";

    private final String mFragmentShader =
        "precision mediump float;\n"                   +
        "varying vec2      vTextureCoord;\n"           +
		"varying vec4      vDiffuseColor;\n"           +
        "uniform sampler2D sTexture;\n"                +
        "void main() {\n"                              +
        "   mediump vec4 color = texture2D(sTexture, vTextureCoord);\n" +
        "   gl_FragColor = vDiffuseColor * color; \n"  +
        "}\n";

    private float[] mMVPMatrix = new float[16];
    private float[] mMMatrix = new float[16];
    private float[] mVMatrix = new float[16];
    private float[] mPMatrix = new float[16];

	private static final int NUM_OF_TEXTURE = 1; 
    
    private int   mProgram;
    private int[] mTextureID = new int[NUM_OF_TEXTURE];
    private int   muMVPMatrixHandle;
    private int   maPositionHandle;
    private int   maTextureHandle;
    private int   maColorHandle;

    private Context mContext;
    private static String TAG = "CubeSampleRenderer";
}
